 #include <P16F876A.INC> ;Processador utilizado
;=====================================================
; Configurao Padro:
; Utilizando crystal de 4 Mhz no projeto
;
; Oscilador 			HS
; Watchdog Timer 		OFF
; Power up timer 		ON
; Brown Out detect 		OFF
; Low Voltage Program	Disable
; Data EE Read Protect  OFF
; Flash Program Write   Write Protection OFF
; Code Protect 			OFF
; Isso em hexadecimal  0x3f32
;=====================================================
 __config 0x3f32
;=====================================================
; Algortimo Generico do Programa
;=====================================================
;
;	Inicia
;   	ConfiguraPic
;       	-Portas  
;           -Serial
;           -Timer para os botes de controle
;		Envia para o pc que esta pronto para receber msg ('r' de ready)
;		Le Msg Da Serial
;       Decofica msg da serial
;		Executa ao da msg
;       Limpa Variaveis utilizadas no processo
;		Volta para Ler Msg da Serial
;
;=====================================================
; Definies de macros
;=====================================================

;Macro de controle do motor para andar para frente ou para tras
motorStepReturn macro testStep,motor,next
  	banksel testStep
    movfw testStep
	movwf temp          ;move o step 1 para w
    xorwf motor,w       
    btfss STATUS,2		;verifica se  igual ou menor
	goto $+4
    movfw next
    movwf motor
    return
    endm
 #include "macros.inc"
;=====================================================
; Variaveis do programa
;=====================================================
 CBLOCK 0x20
	temp			;variavel temporaria de uso diverso
    motor1Step		;proximo passo para o motor 1
    motor2Step      ;proximo passo para o motor 2

    step1           ;passo 1
    step2			;passo 2
	step3           ;passo 3
    step4           ;passo 4

	type_msg		  ;tipo da msg enviada
    isRotEsqPositive  ;indica se o valor do de Rotao  positivo ou negativo
    rotEsq1		      ;Guarda o valor da rotao do motor a esquerda
    rotEsq2

    isRotDirPositive  ;indica se o valor do de Rotao  positivo ou negativo
    rotDir1			  ;Guarda o valor da rotao do motor a direita
	rotDir2			  

	W_TEMP 		; para salvar contexto
    STATUS_TEMP ; para salvar contexto
    PCLATH_TEMP ; para salvar contexto
 ENDC 

 org 0x00
 goto main	  ;programa principal
 org 0x04
 goto isr     ;rotina de tratamento de interrupo

;=====================================================
; Entry Point
;===================================================== 
main:
 call setup			;Inicia o programa configurando o pic			
 call onPowerOnLed  ;Liga led de indicao de atividade
 
 movlw 'h'			;informa o pc que esta vivo
 call _tx
;=====================================================
; Loop Principal
;===================================================== 
loop: 

 ;aqui inicia o processo de recepo e parser da msg
 ;as msgs podem ser do tipo
 ; m para movimento
 ; exemplo: 'm' 'p' 0x00 'p' 0x00 'e'

receive:

 call _rx		   ;Recebe o primeiro caracter
 banksel type_msg
 movwf type_msg	   ;Guarda o valor em uma variavel
 movlw 'm'
 xorwf 	type_msg,w   ; Verifica se a msg  igual a 'm'
 beq mMsgRecebida
 goto receive
 ;----------------------------------------------------------------------
 ;Parte no implementada pois no te controle sobre a caneta ainda
 ;movlw 'l'	       
 ;xorwf 	type_msg,w   ; Verifica se a msg  igual a levantar a caneta
 ;beq subirCanetaMsgRecebida

 ;movlw 'a'	       
 ;xorwf 	type_msg,w   ; Verifica se a msg  igual a descer a caneta
 ;beq subirCanetaMsgRecebida
 ;----------------------------------------------------------------------

;Rotina que recebe a msg de movimento e guarda em variaveis, para controle do movimento 
mMsgRecebida:
 banksel isRotEsqPositive
 call _rx		;recebe dados para o primeiro motor
 movwf isRotEsqPositive
 call _rx		
 movwf rotEsq2
 call _rx		
 movwf rotEsq1

 banksel isRotDirPositive
 call _rx		;recebe dados para o segundo motor
 movwf isRotDirPositive
 call _rx		
 movwf rotDir2
 call _rx		
 movwf rotDir1

 call moveMotorsByMsg	;Move os motores segundo a msg recebida

 goto receive

;=====================================================
; Rotina de configurao
;=====================================================
setup:
;Define a PORTB como saida
 banksel TRISB
 clrf TRISB
 ;movlw 0xFF
 ;movwf TRISA ;define porta como entrada que  a porta que move o 
 
;define os passos usando passo completo( maior torque )
 banksel step1
 movlw .3
 movwf step1
 
 banksel step2
 movlw .6
 movwf step2

 banksel step3
 movlw .12 
 movwf step3

 banksel step4
 movlw .9
 movwf step4

 ; os motores vo para o passo 1
 banksel step1
 movfw step1
 banksel motor1Step 
 movwf motor1Step
 banksel motor2Step
 movwf motor2Step
 call passToPortB; passa para portb os valores para travar no inicio

 ;configura o PORTA como entrada digital
 banksel ADCON1
 MOVLW 0x07   
 MOVWF ADCON1  

 call habilitaIntTimer2

 ;define portc.0 como saida do led
 banksel TRISC
 movfw TRISC
 andlw b'11111110'
 movwf TRISC

 ;configura a interface serial para recebimento
 BANKSEL TXSTA
 movlw b'00100000'
 movwf TXSTA
 movlw .25	;para a velocidade de 2400 bpss em 4 Mhz
 movwf SPBRG
    
 BANKSEL RCSTA
 movlw b'10000000'
 movwf RCSTA
 bsf RCSTA,CREN          ;habilita recepo continua
 return

;=====================================================
; ISR (Interrupt Service Routine)
;=====================================================
isr:
	;Descrio do salvamento de contexto
	;Primeiramente salva o registrador de trabalho W, pois sera utilizado posteriormente
	;MOVWF no afeta registrador de status
	
 	MOVWF W_TEMP      ;Copy W to W_TEMP register
	SWAPF STATUS,W    ;Swap status to be saved into W
	CLRF STATUS       ;bank 0, regardless of current bank, Clears IRP,RP1,RP0
	MOVWF STATUS_TEMP ;Save status to bank zero STATUS_TEMP register
	MOVF PCLATH, W    ;Only required if using pages 1, 2 and/or 3
	MOVWF PCLATH_TEMP ;Save PCLATH into W
	CLRF PCLATH       ;Page zero, regardless of current page

	;desliga interrupes
	banksel INTCON
	bcf INTCON,GIE
	;abaixa o flag de interrupo
	banksel	PIR1
	bcf PIR1,1

	banksel PORTA
	btfsc PORTA,1
    goto primeiro
	btfsc PORTA,2
    goto segundo
	
testMotor2:
    btfsc PORTA,3
    goto terceiro
	btfsc PORTA,4
    goto quarto
	goto termina

;Neste caso para passar para o portb no tem delay pois a interrupo j define um tempo para deixar o motor
;rodar.
primeiro:
	call motorEsquerdaPasso
	call passToPortB
    goto testMotor2
segundo:
    call motorEsquerdaAntiPasso
    call passToPortB
	goto testMotor2
terceiro:
	call motorDireitaPasso
	call passToPortB
    goto termina
quarto:	
	call motorDireitaAntiPasso 
	call passToPortB

termina:
	
	bcf PIR1,TMR2IF      ;limpa o flag de interrupo do timer
	MOVF PCLATH_TEMP, W ;Restore PCLATH
	MOVWF PCLATH        ;Move W into PCLATH
	SWAPF STATUS_TEMP,W ;Swap STATUS_TEMP register into W ;(sets bank to original state)
	MOVWF STATUS        ;Move W into STATUS register
	SWAPF W_TEMP,F      ;Swap W_TEMP
	SWAPF W_TEMP,W      ;Swap W_TEMP into W
	
 RETFIE



;=====================================================
; Habilita a interrupo do timer 2
;=====================================================
habilitaIntTimer2:
  ;Entrada este em 4Mhz 
  ;Entrada do clock  /4 = 1 Mhz
  ;Prescaler usando divisor de 1:16 = 62500 hz
  ;Contador usado sera de 250 para termos um valor de 250Hz
  ;Configurando o pos scaler para 5: para termos 50hz
  banksel T2CON;
  movlw b'0101110';
  movwf T2CON;
  ;configurando o contador para 1
  banksel PR2
  movlw d'250'
  movwf PR2

  banksel PIE1;
  ;bit 1 TMR2IE: TMR2 to PR2 Match Interrupt Enable bit
  ;				 1 = Enables the TMR2 to PR2 match interrupt
  ;			     0 = Disables the TMR2 to PR2 match interrupt
  bsf PIE1,TMR2IE;	  
  
  ;INTCON REGISTER
  ;bit 7: GIE: Global Interrupt Enable bit
  ;	      1 = Enables all un-masked interrupts
  ;	      0 = Disables all interrupts
  ;bit 6 PEIE: Peripheral Interrupt Enable bit
  ;       1 = Enables all unmasked peripheral interrupts
  ;       0 = Disables all peripheral interrupts
  ;---------------------------------------------------
               ;intcon esta no mesmo banco
  ;habilita a interrupo dos perifericos(TIMER2) e a mascara geral de interrupes
  banksel INTCON
  bsf INTCON,PEIE;
  bsf INTCON,GIE;
  
  return  

;=====================================================
; Rotinas
;=====================================================
;move os motores segundo as msg recebidas
moveMotorsByMsg:
 ;se o valor da direita e da esquerda for  0x00 e 0x00 sai da funo
 
 ;Verificar se  zero ser for vai para o proximo
 call rotEsqZero
 BTFSC STATUS,Z
 goto proximo

 ;Verifica se  positivo ou negativo e move o motor
 banksel isRotEsqPositive
 movlw 'p'
 xorwf isRotEsqPositive,w
 beq esqPositive
 goto esqNegative
esqPositive:
 call decrementaMotorEsquerda
 ;gira o motor para direita
 call motorEsquerdaPasso
 goto proximo 	  
esqNegative:
 ;diminui o valor em 1 
 call decrementaMotorEsquerda
 ;gira o motor para esquerda
 call motorEsquerdaAntiPasso  	

proximo:
;Verifica se  zero, ser for vai para final caso contrario continua
 call rotDirZero
 BTFSC STATUS,Z
 goto finalRotacao

 ;Verifica se  positivo ou negativo e move o motor
 banksel isRotDirPositive
 movlw 'p'
 xorwf isRotDirPositive,w
 beq dirPositive
 goto dirNegative
dirPositive:
  ;diminui o valor em 1
 call decrementaMotorDireita    	  
  ;gira o motor para a direita
 call motorDireitaPasso
 goto finalRotacao  	  
dirNegative:
 ;diminui o valor em 1
 call decrementaMotorDireita    	   
 ;gira o motor para a esquerda
 call motorDireitaAntiPasso  

 
finalRotacao:	  
 call passToPortB
 call delay
 call delay
 call delay
 call delay
 call delay
 call delay
 call delay
 call delay
 call delay

 ;se algum motor ainda tem que rodar volta para inicio
 call rotEsqZero
 btfss STATUS,Z
 goto moveMotorsByMsg
 
 call rotDirZero
 btfss STATUS,Z
 goto moveMotorsByMsg

 return

;informa pelo zero flag se esquerda  zero
rotEsqZero:
  movfw rotEsq1		;se  zero pula
  BTFSS STATUS,Z
  return
  movfw rotEsq2 
  return

;informa pelo zero flag se direita  zero
rotDirZero:
  movfw rotDir1		;se  zero pula
  BTFSS STATUS,Z
  return
  movfw rotDir2 
  return

; Rotinas de movimento dos motores
; Da um passo para frente no motor de passo da esquerda
motorEsquerdaPasso:	
    motorStepReturn step1,motor1Step,step2
    motorStepReturn step2,motor1Step,step3
    motorStepReturn step3,motor1Step,step4
    motorStepReturn step4,motor1Step,step1
	return
; Da um passo para tras do motor de passo da esquerda
motorEsquerdaAntiPasso:
	motorStepReturn step1,motor1Step,step4
    motorStepReturn step2,motor1Step,step1
    motorStepReturn step3,motor1Step,step2
    motorStepReturn step4,motor1Step,step3
	return
; Da um passo para frente do motor de passo da direita
motorDireitaPasso:
    motorStepReturn step1,motor2Step,step2
    motorStepReturn step2,motor2Step,step3
    motorStepReturn step3,motor2Step,step4
    motorStepReturn step4,motor2Step,step1
	return
; Da um passo para tras do motor de passo da direita
motorDireitaAntiPasso:
	motorStepReturn step1,motor2Step,step4
    motorStepReturn step2,motor2Step,step1
    motorStepReturn step3,motor2Step,step2
    motorStepReturn step4,motor2Step,step3
	return

;Decrementa Motor
decrementaMotorDireita:
 banksel rotDir1
 movlw 0x01
 subwf rotDir1,f
 btfsc STATUS,C   ;verifica se chegou a zero
 return			 
 
 movfw rotDir2	  ;move rotDir2 para w
 btfsc STATUS,Z   ;verifica se  zero
 return			  ;se form no faz nada
 decf rotDir2	  ;caso contrario decrementa um
 return

decrementaMotorEsquerda:
 banksel rotEsq1
 movlw 0x01
 subwf rotEsq1,f
 btfsc STATUS,C   ;verifica se chegou a zero
 return			  
 
 movfw rotEsq2	  ;move rotEsq2 para w
 btfsc STATUS,Z   ;verifica se  zero
 return			  ;se form no faz nada
 decf rotEsq2	  ;caso contrario decrementa um
 return

;Junta e transfere os valores de motor1Step e motor2Step para o PORTB
passToPortB:
 movfw motor1Step
 movwf temp
 swapf temp,f
 movfw motor2Step
 IORWF temp,w
 banksel PORTB
 movwf PORTB

 return
;Rotina de transmiso pela serial
_tx:
    BANKSEL TXSTA
    btfss TXSTA, TRMT
    goto $-1
    BANKSEL RCSTA
    movwf TXREG
    return
;Rotina de recebimento da serial
_rx:
   ; btfsc PIR1, RCIF   ;Para teste, para no ficar travado aqui.
    btfss PIR1, RCIF  ; Real para a utilizao
    goto $-1
    movfw RCREG
    return

;Rotina que acende o led para indicar o funcionamento do pic
onPowerOnLed:
 banksel PORTC
 movlw 0xFF
 movwf PORTC
 return
;Rotina que muda de ligado para deligado e vice versa o led de indicao de funcionamento
;Utilizado para indicar funcionamento do pic
changeLed:
 movfw PORTC
 xorlw 0x01
 movwf PORTC
 return

;rotina para subir a caneta de desenho
subirCaneta:
 ;TODO
 return
;rotina para descer a caneta de desenho
descerCaneta:
 ;TODO 
 return 

;Rotina de delay para tarefas lentas.
delay:
 movlw d'249'
 movwf temp
loop_in:
 decf temp,f
 btfss STATUS,2 ; verifica se chegou a zero
 goto loop_in
 return

delayInW:
 movwf temp
loop2_in:
 decf temp,f
 btfss STATUS,2 ; verifica se chegou a zero
 goto loop2_in
 return
 end