;clock atual 14khz aproximadament (RC oscilator)

 #include <P16F876A.INC>        ; processor specific variable definitions



;=====================================================
; Definies de macros
;=====================================================

motorStepReturn macro testStep,motor,next
  	banksel testStep
    movfw testStep
	movwf temp          ;move o step 1 para w
    xorwf motor,w       
    btfss STATUS,2		;verifica se  igual ou menor
	goto $+4
    movfw next
    movwf motor
    return
    endm
;=====================================================
; Variaveis do programa
;=====================================================
 CBLOCK 0x20
	troca

    temp			;variavel temporaria para uso comum
    
    motor1Step		;proximo passo para o motor 1
    motor2Step      ;proximo passo para o motor 2

    step1           ;passo 1
    step2			;passo 2
	step3           ;passo 3
    step4           ;passo 4
    

    W_TEMP 		; para salvar contexto
    STATUS_TEMP ; para salvar contexto
    PCLATH_TEMP ; para salvar contexto

 ENDC 
 __config 0x3f33
 org 0
 goto main
 org 4
 goto isr     ;interrup service routine
 
main:
 call giraMotores

 banksel troca
 btfss troca,0
 goto normal
 goto gira

gira:
 bcf troca,0
 call giraMotores

normal:
 bsf troca,0
 call setup

loop:         ;fica dormindo enquanto no aperta os botes
 nop
 goto loop

giraMotores:
;define os passos usando passo completo( maior torque )
 movlw .3 
 movwf step1
 movlw .6
 movwf step2
 movlw .12 
 movwf step3
 movlw .9
 movwf step4
 ; os motores vo para o passo 1
 banksel step1
 movfw step1
 banksel motor1Step 
 movwf motor1Step
 banksel motor2Step
 movwf motor2Step

lgira:
 call motorEsquerdaPasso
 call delay
 call motorDireitaPasso
 call delay
 call passToPortB
 call delay
 goto lgira
 return


;=====================================================
; Rotina de configurao
;=====================================================
setup:
;Define a PORTB como saida
 banksel TRISB
 clrf TRISB
 movlw 0xFF
 movwf TRISA ;define porta como entrada
 banksel PORTB
 
;define os passos usando passo completo( maior torque )
 movlw .3 
 movwf step1
 movlw .6
 movwf step2
 movlw .12 
 movwf step3
 movlw .9
 movwf step4

 ; os motores vo para o passo 1
 banksel step1
 movfw step1
 banksel motor1Step 
 movwf motor1Step
 banksel motor2Step
 movwf motor2Step

 call passToPortB; passa para portb os valores

 ;configura o porta como entrada digital
 banksel ADCON1
 MOVLW 0x06   ; Configure all pins
 MOVWF ADCON1 ; as digital inputs

 call habilitaIntTimer2

 return

isr:
	;desliga interrupes
	banksel INTCON
	bcf INTCON,GIE
;abaixa o flag de interrupo
	banksel	PIR1
	bcf PIR1,1
  	
	;Descrio do salvamento de contexto
	;Primeiramente salva o registrador de trabalho W, pois sera utilizado posteriormente
	;MOVWF no afeta registrador de status
	
 	MOVWF W_TEMP      ;Copy W to W_TEMP register
	SWAPF STATUS,W    ;Swap status to be saved into W
	CLRF STATUS       ;bank 0, regardless of current bank, Clears IRP,RP1,RP0
	MOVWF STATUS_TEMP ;Save status to bank zero STATUS_TEMP register
	MOVF PCLATH, W    ;Only required if using pages 1, 2 and/or 3
	MOVWF PCLATH_TEMP ;Save PCLATH into W
	CLRF PCLATH       ;Page zero, regardless of current page


	banksel PORTA
	btfsc PORTA,1
    goto primeiro
	btfsc PORTA,2
    goto segundo
	btfsc PORTA,3
    goto terceiro
	btfsc PORTA,4
    goto quarto
	goto termina

primeiro:
	call motorEsquerdaPasso
	call passToPortB
    goto termina
segundo:
    call motorEquerdaAntiPasso
    call passToPortB
	goto termina
terceiro:
	call motorDireitaPasso
	call passToPortB
    goto termina
quarto:	
	call motorDireitaAntiPasso 
	call passToPortB

termina:


	bcf PIR1,TMR2IF      ;limpa o flag de interrupo do timer
	MOVF PCLATH_TEMP, W ;Restore PCLATH
	MOVWF PCLATH        ;Move W into PCLATH
	SWAPF STATUS_TEMP,W ;Swap STATUS_TEMP register into W ;(sets bank to original state)
	MOVWF STATUS        ;Move W into STATUS register
	SWAPF W_TEMP,F      ;Swap W_TEMP
	SWAPF W_TEMP,W      ;Swap W_TEMP into W
	
 RETFIE

motorEsquerdaPasso:
    motorStepReturn step1,motor1Step,step2
    motorStepReturn step2,motor1Step,step3
    motorStepReturn step3,motor1Step,step4
    motorStepReturn step4,motor1Step,step1
	return
motorEquerdaAntiPasso:
	motorStepReturn step1,motor1Step,step4
    motorStepReturn step2,motor1Step,step1
    motorStepReturn step3,motor1Step,step2
    motorStepReturn step4,motor1Step,step3
	return
motorDireitaPasso:
    motorStepReturn step1,motor2Step,step2
    motorStepReturn step2,motor2Step,step3
    motorStepReturn step3,motor2Step,step4
    motorStepReturn step4,motor2Step,step1
	return
motorDireitaAntiPasso:
	motorStepReturn step1,motor2Step,step4
    motorStepReturn step2,motor2Step,step1
    motorStepReturn step3,motor2Step,step2
    motorStepReturn step4,motor2Step,step3
	return


;=====================================================
; Habilita a interrupo do timer 2
;=====================================================
habilitaIntTimer2:
  ;Entrada este em 14Khz 
  ;Entrada do clock  /4 = 3,5 khz
  ;Prescaler usando divisor de 1:16 = 218,75 hz
  ;Contador usado sera de 1 para termos um tempo rapido para leitura de teclas
    
  ;Configurando o prescaler para 1:16 e deixa ele em modo para ligar
  banksel T2CON;
  movlw b'0000110';
  movwf T2CON;
  ;configurando o contador para 1
  banksel PR2
  movlw d'1'
  movwf PR2

  banksel PIE1;
  ;bit 1 TMR2IE: TMR2 to PR2 Match Interrupt Enable bit
  ;				 1 = Enables the TMR2 to PR2 match interrupt
  ;			     0 = Disables the TMR2 to PR2 match interrupt
  bsf PIE1,TMR2IE;	  
  
  ;INTCON REGISTER
  ;bit 7: GIE: Global Interrupt Enable bit
  ;	      1 = Enables all un-masked interrupts
  ;	      0 = Disables all interrupts
  ;bit 6 PEIE: Peripheral Interrupt Enable bit
  ;       1 = Enables all unmasked peripheral interrupts
  ;       0 = Disables all peripheral interrupts
  ;---------------------------------------------------
               ;intcon esta no mesmo banco
  ;habilita a interrupo dos perifericos(TIMER2) e a mascara geral de interrupes
  banksel INTCON
  bsf INTCON,PEIE;
  bsf INTCON,GIE;
  
  return  


passToPortB:
 movfw motor1Step
 movwf temp
 swapf temp,f
 movfw motor2Step
 IORWF temp,w
 banksel PORTB
 movwf PORTB

 return

delay:
 movlw d'200'
 movwf temp
loop_in:
 decf temp
 btfss STATUS,2 ; verifica se chegou a zero
 goto loop_in
 return
 end

