	list      p=16F876A            ; list directive to define processor
	#include <P16F876A.INC>        ; processor specific variable definitions
    
	__CONFIG 0x3F32

	CBLOCK 0x20
	temp
	w_temp : 1        ; variable used for context saving 
	status_temp : 1   ; variable used for context saving
	ENDC

	ORG     0x000             ; processor reset vector
    clrf    PCLATH            ; ensure page bits are cleared
    goto    setup              ; go to beginning of program

	ORG     0x004             ; interrupt vector location
    movwf   w_temp            ; save off current W register contents
    movf    STATUS,w          ; move status register into W register
    movwf   status_temp       ; save off contents of STATUS register

    movf    status_temp,w     ; retrieve copy of STATUS register
    movwf   STATUS            ; restore pre-isr STATUS register contents
    swapf   w_temp,f
    swapf   w_temp,w          ; restore pre-isr W register contents
    retfie                    ; return from interrupt



setup:
    BANKSEL TRISB ; configurando o portb como saida
    clrf TRISB
	banksel TRISC ; configurando o rc7 como entrada e rc6 como saida
	movlw 0xBF
	movwf TRISC	

    BANKSEL TXSTA
    movlw b'00100000'
    movwf TXSTA
    movlw .51	;para a velocidade de 4800 bps em 4 Mhz
    movwf SPBRG
    
    BANKSEL RCSTA
    movlw b'10000000'
    movwf RCSTA
	BSF	RCSTA,CREN          ;habilita recepo continua

main:
  	banksel PORTB
    movlw 0x01
    movwf PORTB

loop:
    banksel SPBRG
	movfw SPBRG
    banksel TXREG
	movwf TXREG
    call _tx
	call delay
	call delay
	call delay
	call changeLed
	banksel SPBRG
	goto loop
  

_tx:
    BANKSEL TXSTA
    btfss TXSTA, TRMT
    goto $-1
    BANKSEL RCSTA
    movwf TXREG
    return

_rx:
    btfsc PIR1, RCIF
    goto $-1
    movfw RCREG
    return

changeLed:
	movfw PORTB
	xorlw 0x01
	movwf PORTB
	return
delay:
	movlw d'250'
	movwf temp
loop_in:
	decf temp
	btfss STATUS,2 ; verifica se chegou a zero
	goto loop_in
	return    
    END
